#ifndef RANDOM_H
#define RANDOM_H

#include <common/BasicTypes.h>

#include "MersenneTwister.h"

// simple wrapper around the Mersenne Twister
// hides a bunch of options and renames functions to be nicer
class Random
{
public:

	Random(); // from /dev/random or from clock()
	Random( const uint& seed ); // seed from integer
	// TODO: seed from array?

	// [0,1]
	double nextDouble() const;

	// [0, 2^32 - 1]
	uint nextInt() const;

	// [0,n] for n < 2^32
	int nextIntInclusive( const uint& n ) const;

	// [0,1)
	double zeroInclusiveOneExclusive() const;

	// [0,1]
	double zeroInclusiveOneInclusive() const;

	// [0,n)
	double zeroInclusiveNExclusive( const double& n ) const;

	// [0,n]
	double zeroInclusiveNInclusive( const double& n ) const;

	// (0,1)
	double zeroExclusiveOneExclusive() const;

	// (0,n)
	double zeroExclusiveNExclusive( const double& n ) const;	

private:

	mutable MTRand m_mtRand;

};

#endif // RANDOM_H
