#ifndef GMATRIXD_H
#define GMATRIXD_H

#include <common/BasicTypes.h>
#include <common/ReferenceCountedArray.h>

class GMatrixd
{
public:

	GMatrixd(); // makes a 0x0 matrix
	GMatrixd( uint nRows, uint nCols, double fillValue = 0 );
	GMatrixd( const GMatrixd& m );
	virtual ~GMatrixd();
	GMatrixd& operator = ( const GMatrixd& m );

	bool isNull() const; // a matrix is null if either nRows or nCols is 0

	void fill( double d );

	uint numRows() const;
	uint numCols() const;
	uint numElements() const;
	void indexToSubscript( uint idx, uint* pi, uint* pj ) const;
	uint subscriptToIndex( uint i, uint j ) const;

	void resize( uint nRows, uint nCols );

	void reshape( uint nRows, uint nCols );

	// access at ( i, j )
	double& operator () ( uint i, uint j );
	const double& operator () ( uint i, uint j ) const;

	// access element k with column-major ordering
	double& operator [] ( uint k );
	const double& operator [] ( uint k ) const;

	DoubleArray data() const;

private:

	uint m_nRows;
	uint m_nCols;
	DoubleArray m_aData;
};

#endif // GMATRIXD_H
