#ifndef VECTOR_2F_H
#define VECTOR_2F_H

class Vector2d;
class Vector2i;
class Vector3f;


class Vector2f
{
public:

	// TODO: conversion operators for double <--> float on Vector3f and Vector4f

	Vector2f();
	Vector2f( float x, float y );

	// copy constructors
	Vector2f( const Vector2f& rv );
	Vector2f( const Vector2d& rv );
	Vector2f( const Vector2i& rv );

	// assignment operators
	Vector2f& operator = ( const Vector2f& rv );
	Vector2f& operator = ( const Vector2d& rv );
	Vector2f& operator = ( const Vector2i& rv );

	// no destructor necessary

	// returns the ith element (mod 2)
	const float& operator [] ( int i ) const;
	float& operator [] ( int i );

	float x() const;
	float y() const;
	Vector2f xy() const;
	Vector2f yx() const;
	Vector2f xx() const;
	Vector2f yy() const;

	float abs() const;
	float absSquared() const;
	void normalize();
	Vector2f normalized() const;

	void negate();

	// ---- Utility ----
	operator const float* (); // automatic type conversion for GL
	void print() const;

	static float dot( const Vector2f& v0, const Vector2f& v1 );	

	static Vector3f cross( const Vector2f& v0, const Vector2f& v1 );

	// returns v0 * ( 1 - alpha ) * v1 * alpha
	static Vector2f lerp( const Vector2f& v0, const Vector2f& v1, float alpha );

private:

	float m_afElements[2];

};

Vector2f operator + ( const Vector2f& v0, const Vector2f& v1 );
Vector2f operator - ( const Vector2f& v0, const Vector2f& v1 );
Vector2f operator * ( const Vector2f& v0, const Vector2f& v1 );
Vector2f operator / ( const Vector2f& v0, const Vector2f& v1 );

Vector2f operator - ( const Vector2f& v );
Vector2f operator * ( float f, const Vector2f& v );
Vector2f operator * ( const Vector2f& v, float f );

#endif // VECTOR_2F_H
