#include "Cg/CgProgramManager.h"

#include <cassert>

#include "Cg/CgProgramWrapper.h"
#include "Cg/CgShared.h"

//////////////////////////////////////////////////////////////////////////
// Public
//////////////////////////////////////////////////////////////////////////

// static
CgProgramManager* CgProgramManager::getInstance()
{
	if( s_pSingleton == NULL )
	{
		s_pSingleton = new CgProgramManager;
	}
	return s_pSingleton;
}

// static
void CgProgramManager::destroy()
{
	if( s_pSingleton != NULL )
	{
		delete s_pSingleton;
	}
}

void CgProgramManager::loadProgramFromFile( QString programName,
										  QString filename, QString entryPoint,
										  CGprofile profile,
										  const char** args )
{
	CGcontext context;
	context = CgShared::getInstance()->getSharedCgContext();

	CgProgramWrapper* pProgram = CgProgramWrapper::create( filename, entryPoint, context, profile );	
	assert( pProgram != NULL );

	if( m_qhPrograms.contains( programName ) )
	{
		delete( m_qhPrograms.take( programName ) );
	}

	m_qhPrograms[ programName ] = pProgram;
}

CgProgramWrapper* CgProgramManager::getNamedProgram( QString programName )
{
	if( m_qhPrograms.contains( programName ) )
	{
		return m_qhPrograms[ programName ];
	}
	else
	{
		assert( false );
		return NULL;
	}
}

void CgProgramManager::destroyNamedProgram( QString programName )
{
	if( m_qhPrograms.contains( programName ) )
	{
		delete( m_qhPrograms.take( programName ) );
	}
}

//////////////////////////////////////////////////////////////////////////
// Private
//////////////////////////////////////////////////////////////////////////

CgProgramManager::CgProgramManager()
{

}

// virtual
CgProgramManager::~CgProgramManager()
{
	foreach( QString key, m_qhPrograms.keys() )
	{
		delete m_qhPrograms.take( key );		
	}
}

// static
CgProgramManager* CgProgramManager::s_pSingleton = NULL;
