#include "Cg/CgShared.h"

#include <cstdio> // for stderr
#include <cstdlib> // just for NULL

//////////////////////////////////////////////////////////////////////////
// Global
//////////////////////////////////////////////////////////////////////////

void cgSharedErrorHandler( CGcontext ctx, CGerror err, void* appdata )
{
	fprintf( stderr, "Cg error: %s\n", cgGetErrorString( err ) );
	const char* listing = cgGetLastListing( ctx );
	if( listing != NULL )
	{
		fprintf( stderr, "    last listing: %s\n", listing );
	}
}

//////////////////////////////////////////////////////////////////////////
// Public
//////////////////////////////////////////////////////////////////////////

// static
CgShared* CgShared::getInstance()
{
	if( s_pSingleton == NULL )
	{
		s_pSingleton = new CgShared;
	}

	return s_pSingleton;
}

// static
void CgShared::destroy()
{
	if( s_pSingleton != NULL )
	{
		delete s_pSingleton;
		s_pSingleton = NULL;
	}
}

CGcontext CgShared::getSharedCgContext() const
{
	return m_cgContext;
}

CGprofile CgShared::getLatestVertexProfile() const
{
	return m_cgVertexProfile;
}

CGprofile CgShared::getLatestFragmentProfile() const
{
	return m_cgFragmentProfile;
}

//////////////////////////////////////////////////////////////////////////
// Private
//////////////////////////////////////////////////////////////////////////

CgShared::CgShared()
{
	m_cgContext = cgCreateContext();
	cgSetAutoCompile( m_cgContext, CG_COMPILE_IMMEDIATE );
	cgGLRegisterStates( m_cgContext );
	cgGLSetManageTextureParameters( m_cgContext, CG_TRUE );

	cgSetErrorHandler( cgSharedErrorHandler, NULL );

	m_cgVertexProfile = cgGLGetLatestProfile( CG_GL_VERTEX );
	cgGLSetOptimalOptions( m_cgVertexProfile );

	m_cgFragmentProfile = cgGLGetLatestProfile( CG_GL_FRAGMENT );
	cgGLSetOptimalOptions( m_cgFragmentProfile );
}

// virtual
CgShared::~CgShared()
{
	cgDestroyContext( m_cgContext );
}

CgShared* CgShared::s_pSingleton = NULL;
