#include "geometry/BoundingBox3f.h"

#include <algorithm>
#include <cfloat>
#include <cstdio>

using namespace std;

BoundingBox3f::BoundingBox3f() :
	
	m_min( FLT_MAX, FLT_MAX, FLT_MAX ),
	m_max( FLT_MIN, FLT_MIN, FLT_MIN )

{

}

BoundingBox3f::BoundingBox3f( float minX, float minY, float minZ,
							 float maxX, float maxY, float maxZ ) :

	m_min( minX, minY, minZ ),
	m_max( maxX, maxY, maxZ )

{

}

BoundingBox3f::BoundingBox3f( const Vector3f& min, const Vector3f& max ) :

	m_min( min ),
	m_max( max )

{

}

BoundingBox3f::BoundingBox3f( const BoundingBox3f& rb ) :

	m_min( rb.m_min ),
	m_max( rb.m_max )

{

}

BoundingBox3f& BoundingBox3f::operator = ( const BoundingBox3f& rb )
{
	if( this != &rb )
	{
		m_min = rb.m_min;
		m_max = rb.m_max;
	}
	return *this;
}

void BoundingBox3f::print()
{
	printf( "min: " );
	m_min.print();
	printf( "max: " );
	m_max.print();
}

Vector3f& BoundingBox3f::minimum()
{
	return m_min;
}

Vector3f& BoundingBox3f::maximum()
{
	return m_max;
}

Vector3f BoundingBox3f::minimum() const
{
	return m_min;
}

Vector3f BoundingBox3f::maximum() const
{
	return m_max;
}

Vector3f BoundingBox3f::range() const
{
	return( m_max - m_min );
}

Vector3f BoundingBox3f::center() const
{
	return( 0.5 * ( m_max + m_min ) );
}

bool BoundingBox3f::overlaps( const BoundingBox3f& other )
{
	bool bOverlapsInDirection[3];

	Vector3f otherMin = other.minimum();
	Vector3f otherMax = other.maximum();

	for( int i = 0; i < 3; ++i )
	{
		bool bMinInside0 = ( otherMin[i] >= m_min[i] ) && ( otherMin[i] <= m_max[i] );
		bool bMinInside1 = ( m_min[i] >= otherMin[i] ) && ( m_min[i] <= otherMax[i] );

		bool bMaxInside0 = ( otherMax[i] >= m_min[i] ) && ( otherMax[i] <= m_max[i] );
		bool bMaxInside1 = ( m_max[i] >= otherMin[i] ) && ( m_max[i] <= otherMax[i] );

		bool bMinInside = bMinInside0 || bMinInside1;
		bool bMaxInside = bMaxInside0 || bMaxInside1;

		bOverlapsInDirection[i] = bMinInside || bMaxInside;
	}

	return bOverlapsInDirection[0] && bOverlapsInDirection[1] && bOverlapsInDirection[2];
}

// static
BoundingBox3f BoundingBox3f::merge( const BoundingBox3f& b0, const BoundingBox3f& b1 )
{
	Vector3f b0Min = b0.minimum();
	Vector3f b0Max = b0.maximum();
	Vector3f b1Min = b1.minimum();
	Vector3f b1Max = b1.maximum();

	Vector3f mergedMin( min( b0Min.x(), b1Min.x() ), min( b0Min.y(), b1Min.y() ), min( b0Min.z(), b1Min.z() ) );
	Vector3f mergedMax( max( b0Max.x(), b1Max.x() ), max( b0Max.y(), b1Max.y() ), max( b0Max.z(), b1Max.z() ) );

	return BoundingBox3f( mergedMin, mergedMax );
}
