#include <cmath>
#include <cstdio>
#include <cstdlib>

#include "vecmath/Vector4d.h"
#include "vecmath/Vector2d.h"
#include "vecmath/Vector3d.h"

Vector4d::Vector4d()
{
	m_adElements[0] = 0;
	m_adElements[1] = 0;
	m_adElements[2] = 0;
	m_adElements[3] = 0;
}

Vector4d::Vector4d( double x, double y, double z, double w )
{
	m_adElements[0] = x;
	m_adElements[1] = y;
	m_adElements[2] = z;
	m_adElements[3] = w;
}

Vector4d::Vector4d( const Vector2d& xy, double z, double w )
{
	m_adElements[0] = xy.x();
	m_adElements[1] = xy.y();
	m_adElements[2] = z;
	m_adElements[3] = w;
}

Vector4d::Vector4d( double x, const Vector2d& yz, double w )
{
	m_adElements[0] = x;
	m_adElements[1] = yz.x();
	m_adElements[2] = yz.y();
	m_adElements[3] = w;
}

Vector4d::Vector4d( double x, double y, const Vector2d& zw )
{
	m_adElements[0] = x;
	m_adElements[1] = y;
	m_adElements[2] = zw.x();
	m_adElements[3] = zw.y();
}

Vector4d::Vector4d( const Vector2d& xy, const Vector2d& zw )
{
	m_adElements[0] = xy.x();
	m_adElements[1] = xy.y();
	m_adElements[2] = zw.x();
	m_adElements[3] = zw.y();
}

Vector4d::Vector4d( const Vector3d& xyz, double w )
{
	m_adElements[0] = xyz.x();
	m_adElements[1] = xyz.y();
	m_adElements[2] = xyz.z();
	m_adElements[3] = w;
}

Vector4d::Vector4d( double x, const Vector3d& yzw )
{
	m_adElements[0] = x;
	m_adElements[1] = yzw.x();
	m_adElements[2] = yzw.y();
	m_adElements[3] = yzw.z();
}

Vector4d::Vector4d( const Vector4d& rv )
{
	m_adElements[0] = rv.m_adElements[0];
	m_adElements[1] = rv.m_adElements[1];
	m_adElements[2] = rv.m_adElements[2];
	m_adElements[3] = rv.m_adElements[3];
}

Vector4d& Vector4d::operator = ( const Vector4d& rv )
{
	if( this != &rv )
	{
		m_adElements[0] = rv.m_adElements[0];
		m_adElements[1] = rv.m_adElements[1];
		m_adElements[2] = rv.m_adElements[2];
		m_adElements[3] = rv.m_adElements[3];
	}
	return *this;
}

const double& Vector4d::operator [] ( int i ) const
{
	return m_adElements[ i % 4 ];
}

double& Vector4d::operator [] ( int i )
{
	return m_adElements[ i % 4 ];
}

double Vector4d::x() const
{
	return m_adElements[0];
}

double Vector4d::y() const
{
	return m_adElements[1];
}

double Vector4d::z() const
{
	return m_adElements[2];
}

double Vector4d::w() const
{
	return m_adElements[3];
}

Vector2d Vector4d::xy() const
{
	return Vector2d( m_adElements[0], m_adElements[1] );
}

Vector2d Vector4d::yz() const
{
	return Vector2d( m_adElements[1], m_adElements[2] );
}

Vector2d Vector4d::zw() const
{
	return Vector2d( m_adElements[2], m_adElements[3] );
}

Vector2d Vector4d::wx() const
{
	return Vector2d( m_adElements[3], m_adElements[0] );
}

Vector3d Vector4d::xyz() const
{
	return Vector3d( m_adElements[0], m_adElements[1], m_adElements[2] );
}

Vector3d Vector4d::yzw() const
{
	return Vector3d( m_adElements[1], m_adElements[2], m_adElements[3] );
}

Vector3d Vector4d::zwx() const
{
	return Vector3d( m_adElements[2], m_adElements[3], m_adElements[0] );
}

Vector3d Vector4d::wxy() const
{
	return Vector3d( m_adElements[3], m_adElements[0], m_adElements[1] );
}

Vector3d Vector4d::xyw() const
{
	return Vector3d( m_adElements[0], m_adElements[1], m_adElements[3] );
}

Vector3d Vector4d::yzx() const
{
	return Vector3d( m_adElements[1], m_adElements[2], m_adElements[0] );
}

Vector3d Vector4d::zwy() const
{
	return Vector3d( m_adElements[2], m_adElements[3], m_adElements[1] );
}

Vector3d Vector4d::wxz() const
{
	return Vector3d( m_adElements[3], m_adElements[0], m_adElements[2] );
}

double Vector4d::abs() const
{
	return sqrt( m_adElements[0] * m_adElements[0] + m_adElements[1] * m_adElements[1] + m_adElements[2] * m_adElements[2] + m_adElements[3] * m_adElements[3] );
}

double Vector4d::absSquared() const
{
	return( m_adElements[0] * m_adElements[0] + m_adElements[1] * m_adElements[1] + m_adElements[2] * m_adElements[2] + m_adElements[3] * m_adElements[3] );
}

void Vector4d::normalize()
{
	double norm = sqrt( m_adElements[0] * m_adElements[0] + m_adElements[1] * m_adElements[1] + m_adElements[2] * m_adElements[2] + m_adElements[3] * m_adElements[3] );
	m_adElements[0] = m_adElements[0] / norm;
	m_adElements[1] = m_adElements[1] / norm;
	m_adElements[2] = m_adElements[2] / norm;
	m_adElements[3] = m_adElements[3] / norm;
}

Vector4d Vector4d::normalized() const
{
	double length = abs();
	return Vector4d
		(
			m_adElements[0] / length,
			m_adElements[1] / length,
			m_adElements[2] / length,
			m_adElements[3] / length
		);
}

void Vector4d::homogenize()
{
	if( m_adElements[3] != 0 )
	{
		m_adElements[0] /= m_adElements[3];
		m_adElements[1] /= m_adElements[3];
		m_adElements[2] /= m_adElements[3];
		m_adElements[3] = 1;
	}
}

Vector4d Vector4d::homogenized() const
{
	if( m_adElements[3] != 0 )
	{
		return Vector4d
			(
				m_adElements[0] / m_adElements[3],
				m_adElements[1] / m_adElements[3],
				m_adElements[2] / m_adElements[3],
				1
			);
	}
	else
	{
		return Vector4d
			(
				m_adElements[0],
				m_adElements[1],
				m_adElements[2],
				m_adElements[3]
			);
	}
}

void Vector4d::negate()
{
	m_adElements[0] = -m_adElements[0];
	m_adElements[1] = -m_adElements[1];
	m_adElements[2] = -m_adElements[2];
	m_adElements[3] = -m_adElements[3];
}

// ---- Utility ----

Vector4d::operator const double* ()
{
	return m_adElements;
}

void Vector4d::print() const
{
	printf( "< %1.2lf, %1.2lf, %1.2lf, %1.2lf >\n",
		m_adElements[0], m_adElements[1], m_adElements[2], m_adElements[3] );
}

// static
double Vector4d::dot( const Vector4d& v0, const Vector4d& v1 )
{
	return v0.x() * v1.x() + v0.y() * v1.y() + v0.z() * v1.z() + v0.w() * v1.w();
}

// static
Vector4d Vector4d::lerp( const Vector4d& v0, const Vector4d& v1, double alpha )
{
	return alpha * ( v1 - v0 ) + v0;
}

//////////////////////////////////////////////////////////////////////////
// Operators
//////////////////////////////////////////////////////////////////////////

Vector4d operator + ( const Vector4d& v0, const Vector4d& v1 )
{
	return Vector4d( v0.x() + v1.x(), v0.y() + v1.y(), v0.z() + v1.z(), v0.w() + v1.w() );
}

Vector4d operator - ( const Vector4d& v0, const Vector4d& v1 )
{
	return Vector4d( v0.x() - v1.x(), v0.y() - v1.y(), v0.z() - v1.z(), v0.w() - v1.w() );
}

Vector4d operator * ( const Vector4d& v0, const Vector4d& v1 )
{
	return Vector4d( v0.x() * v1.x(), v0.y() * v1.y(), v0.z() * v1.z(), v0.w() * v1.w() );
}

Vector4d operator / ( const Vector4d& v0, const Vector4d& v1 )
{
	return Vector4d( v0.x() / v1.x(), v0.y() / v1.y(), v0.z() / v1.z(), v0.w() / v1.w() );
}

Vector4d operator - ( const Vector4d& v )
{
	return Vector4d( -v.x(), -v.y(), -v.z(), -v.w() );
}

Vector4d operator * ( double d, const Vector4d& v )
{
	return Vector4d( d * v.x(), d * v.y(), d * v.z(), d * v.w() );
}

Vector4d operator * ( const Vector4d& v, double d )
{
	return Vector4d( d * v.x(), d * v.y(), d * v.z(), d * v.w() );
}
